@@ -1,5 +1,5 @@
 package Date::Manip;
-# Copyright (c) 2010-2010 Sullivan Beck.  All rights reserved.
+# Copyright (c) 2010-2013 Sullivan Beck.  All rights reserved.
 # This program is free software; you can redistribute it and/or modify it
 # under the same terms as Perl itself.
 
@@ -11,7 +11,7 @@ use strict;
 use Exporter;
 
 our $VERSION;
-$VERSION='6.14';
+$VERSION='6.39';
 
 our (@ISA,@EXPORT);
 
@@ -30,7 +30,10 @@ if ((exists $ENV{'DATE_MANIP'}  &&  $ENV{'DATE_MANIP'} eq 'DM5') ||
 
 my $backend_exp = $backend . "::EXPORT";
 
-eval "require $backend; $backend->import(); return 1;";
+my $flag = eval "require $backend; $backend->import(); return 'loaded';";
+if (! $flag) {
+   die "ERROR LOADING MODULE";
+}
 
 {
    no strict 'refs';
@@ -48,5 +51,5 @@ unshift (@ISA, $backend);
 # cperl-continued-brace-offset: 0
 # cperl-brace-offset: 0
 # cperl-brace-imaginary-offset: 0
-# cperl-label-offset: -2
+# cperl-label-offset: 0
 # End:
