@@ -1,5 +1,5 @@
 package Date::Manip::Delta;
-# Copyright (c) 1995-2010 Sullivan Beck. All rights reserved.
+# Copyright (c) 1995-2013 Sullivan Beck. All rights reserved.
 # This program is free software; you can redistribute it and/or modify it
 # under the same terms as Perl itself.
 
@@ -17,11 +17,15 @@ use Date::Manip::Obj;
 require 5.010000;
 use warnings;
 use strict;
+use utf8;
 use IO::File;
 #use re 'debug';
 
+use Date::Manip::Base;
+use Date::Manip::TZ;
+
 our $VERSION;
-$VERSION='6.14';
+$VERSION='6.39';
 END { undef $VERSION; }
 
 ########################################################################
@@ -51,13 +55,18 @@ sub _init {
    my $dmt = $$self{'tz'};
    my $dmb = $$dmt{'base'};
 
-   $$self{'err'}              = '';
-   $$self{'data'}{'delta'}    = $def;   # the delta
-   $$self{'data'}{'business'} = 0;      # 1 for a business delta
-   $$self{'data'}{'gotmode'}  = 0;      # if exact/business set explicitly
-   $$self{'data'}{'in'}       = '';     # the string that was parsed (if any)
-   $$self{'data'}{'f'}        = {};     # format fields
-   $$self{'data'}{'flen'}     = {};     # field lengths
+   $$self{'err'}  = '';
+   $$self{'data'} = {
+                     'delta'      => $def,  # the delta (all negative fields signed)
+                     'in'         => '',    # the string that was parsed (if any)
+                     'length'     => 0,     # length of delta (in seconds)
+
+                     'gotmode'    => 0,     # 1 if mode set explicitly
+                     'business'   => 0,     # 1 for a business delta
+
+                     'f'          => {},    # format fields
+                     'flen'       => {},    # field lengths
+                    }
 }
 
 sub _init_args {
@@ -81,10 +90,15 @@ sub value {
    return undef  if ($$self{'err'});
    if (wantarray) {
       return @{ $$self{'data'}{'delta'} };
-   } elsif ($$self{'data'}{'business'}) {
-      return $dmb->join('business',$$self{'data'}{'delta'});
    } else {
-      return $dmb->join('delta',$$self{'data'}{'delta'});
+      my @delta = @{ $$self{'data'}{'delta'} };
+      my $err;
+      ($err,@delta) = $dmb->_delta_fields( { 'nonorm'  => 1,
+                                             'source'  => 'delta',
+                                             'sign'    => 0 },
+                                           [@delta]);
+      return undef  if ($err);
+      return join(':',@delta);
    }
 }
 
@@ -98,35 +112,39 @@ sub input {
 ########################################################################
 
 BEGIN {
-   my %ops = map { $_,1 } qw( delta business normal );
-   my %f   = map { $_,1 } qw( y M w d h m s );
+   my %ops = map { $_,1 } qw( delta business normal standard );
+   my %f   = qw( y 0  M 1  w 2  d 3  h 4  m 5  s 6 );
 
    sub set {
-      my($self,$field,$val) = @_;
+      my($self,$field,$val,$no_normalize) = @_;
+
+      my $dmt        = $$self{'tz'};
+      my $dmb        = $$dmt{'base'};
+      my $zone       = $$self{'data'}{'tz'};
+      my $gotmode    = $$self{'data'}{'gotmode'};
+      my $business   = 0;
 
-      $field       = lc($field);
-      my $business = 0;
-      my $dmt      = $$self{'tz'};
-      my $dmb      = $$dmt{'base'};
-      my $zone     = $$self{'data'}{'tz'};
-      my $gotmode  = $$self{'data'}{'gotmode'};
       my (@delta,$err);
 
-      if (exists $ops{$field}) {
+      if (exists $ops{lc($field)}) {
+         $field       = lc($field);
 
          if ($field eq 'business') {
             $business = 1;
             $gotmode  = 1;
-         } elsif ($field eq 'normal') {
+         } elsif ($field eq 'normal'  ||  $field eq 'standard') {
             $business = 0;
             $gotmode  = 1;
+         } elsif ($field eq 'delta') {
+            $business = $$self{'data'}{'business'};
+            $gotmode  = $$self{'data'}{'gotmode'};
          }
-         my $type = ($business ? 'business' : 'delta');
-         if ($business) {
-            ($err,@delta) = $dmb->_normalize_business('norm',@$val);
-         } else {
-            ($err,@delta) = $dmb->_normalize_delta('norm',@$val);
-         }
+
+         ($err,@delta) = $dmb->_delta_fields( { 'nonorm'   => $no_normalize,
+                                                'business' => $business,
+                                                'source'   => 'delta',
+                                                'sign'     => -1 },
+                                              $val);
 
       } elsif (exists $f{$field}) {
 
@@ -137,22 +155,21 @@ BEGIN {
 
          @delta             = @{ $$self{'data'}{'delta'} };
          $business          = $$self{'data'}{'business'};
-         my %f              = qw(y 0 M 1 w 2 d 3 h 4 m 5 s 6);
          $delta[$f{$field}] = $val;
 
-         if ($business) {
-            ($err,@delta) = $dmb->_normalize_business(0,@delta);
-         } else {
-            ($err,@delta) = $dmb->_normalize_delta(0,@delta);
-         }
+         ($err,@delta) = $dmb->_delta_fields( { 'nonorm'   => $no_normalize,
+                                                'business' => $business,
+                                                'source'   => 'delta',
+                                                'sign'     => -1 },
+                                              [@delta]);
 
-      } elsif ($field eq 'mode') {
+      } elsif (lc($field) eq 'mode') {
 
          @delta             = @{ $$self{'data'}{'delta'} };
-         $val = lc($val);
-         if ($val eq "business"  ||  $val eq "normal") {
-            $gotmode = 1;
-            $business = ($val eq "business" ? 1 : 0);
+         $val               = lc($val);
+         if ($val eq 'business'  ||  $val eq 'normal'  ||  $val eq 'standard') {
+            $gotmode        = 1;
+            $business       = ($val eq 'business' ? 1 : 0);
 
          } else {
             $$self{'err'} = "[set] Invalid mode: $val";
@@ -172,9 +189,11 @@ BEGIN {
       }
 
       $self->_init();
-      $$self{'data'}{'delta'}    = [ @delta ];
-      $$self{'data'}{'business'} = $business;
-      $$self{'data'}{'gotmode'}  = $gotmode;
+      $$self{'data'}{'delta'}      = [ @delta ];
+      $$self{'data'}{'business'}   = $business;
+      $$self{'data'}{'gotmode'}    = $gotmode;
+      $$self{'data'}{'length'}     = 'unknown';
+
       return 0;
    }
 }
@@ -191,13 +210,23 @@ sub _rx {
       my $sign    = '[-+]?\s*';
       my $sep     = '(?:,\s*|\s+|$)';
 
-      my $y       = "(?:(?<y>$sign\\d+)\\s*(?:$$dmb{data}{rx}{fields}[1])$sep)";
-      my $m       = "(?:(?<m>$sign\\d+)\\s*(?:$$dmb{data}{rx}{fields}[2])$sep)";
-      my $w       = "(?:(?<w>$sign\\d+)\\s*(?:$$dmb{data}{rx}{fields}[3])$sep)";
-      my $d       = "(?:(?<d>$sign\\d+)\\s*(?:$$dmb{data}{rx}{fields}[4])$sep)";
-      my $h       = "(?:(?<h>$sign\\d+)\\s*(?:$$dmb{data}{rx}{fields}[5])$sep)";
-      my $mn      = "(?:(?<mn>$sign\\d+)\\s*(?:$$dmb{data}{rx}{fields}[6])$sep)";
-      my $s       = "(?:(?<s>$sign\\d+)\\s*(?:$$dmb{data}{rx}{fields}[7])?)";
+      my $nth     = $$dmb{'data'}{'rx'}{'nth'}[0];
+      my $yf      = $$dmb{data}{rx}{fields}[1];
+      my $mf      = $$dmb{data}{rx}{fields}[2];
+      my $wf      = $$dmb{data}{rx}{fields}[3];
+      my $df      = $$dmb{data}{rx}{fields}[4];
+      my $hf      = $$dmb{data}{rx}{fields}[5];
+      my $mnf     = $$dmb{data}{rx}{fields}[6];
+      my $sf      = $$dmb{data}{rx}{fields}[7];
+      my $num     = '(?:\d+(?:\.\d*)?|\.\d+)';
+
+      my $y       = "(?:(?:(?<y>$sign$num)|(?<y>$nth))\\s*(?:$yf)$sep)";
+      my $m       = "(?:(?:(?<m>$sign$num)|(?<m>$nth))\\s*(?:$mf)$sep)";
+      my $w       = "(?:(?:(?<w>$sign$num)|(?<w>$nth))\\s*(?:$wf)$sep)";
+      my $d       = "(?:(?:(?<d>$sign$num)|(?<d>$nth))\\s*(?:$df)$sep)";
+      my $h       = "(?:(?:(?<h>$sign$num)|(?<h>$nth))\\s*(?:$hf)$sep)";
+      my $mn      = "(?:(?:(?<mn>$sign$num)|(?<mn>$nth))\\s*(?:$mnf)$sep)";
+      my $s       = "(?:(?:(?<s>$sign$num)|(?<s>$nth))\\s*(?:$sf)?)";
 
       my $exprx   = qr/^\s*$y?$m?$w?$d?$h?$mn?$s?\s*$/i;
       $$dmb{'data'}{'rx'}{'delta'}{$rx} = $exprx;
@@ -218,10 +247,61 @@ sub _rx {
 }
 
 sub parse {
-   my($self,$instring,$business) = @_;
+   my($self,$instring,@args) = @_;
+   my($business,$no_normalize,$gotmode,$err,@delta);
+
+   if (@args == 2) {
+      ($business,$no_normalize) = (lc($args[0]),lc($args[1]));
+      if      ($business eq 'standard') {
+         $business = 0;
+      } elsif ($business eq 'business') {
+         $business = 1;
+      } elsif ($business) {
+         $business = 1;
+      } else {
+         $business = 0;
+      }
+      if ($no_normalize) {
+         $no_normalize = 1;
+      } else {
+         $no_normalize = 0;
+      }
+      $gotmode = 1;
+
+   } elsif (@args == 1) {
+      my $arg = lc($args[0]);
+      if      ($arg eq 'standard') {
+         $business     = 0;
+         $no_normalize = 0;
+         $gotmode      = 1;
+      } elsif ($arg eq 'business') {
+         $business     = 1;
+         $no_normalize = 0;
+         $gotmode      = 1;
+      } elsif ($arg eq 'nonormalize') {
+         $business     = 0;
+         $no_normalize = 1;
+         $gotmode      = 0;
+      } elsif ($arg) {
+         $business     = 1;
+         $no_normalize = 0;
+         $gotmode      = 1;
+      } else {
+         $business     = 0;
+         $no_normalize = 0;
+         $gotmode      = 0;
+      }
+   } elsif (@args == 0) {
+      $business     = 0;
+      $no_normalize = 0;
+      $gotmode      = 0;
+   } else {
+      $$self{'err'} = "[parse] Unknown arguments";
+      return 1;
+   }
+
    my $dmt = $$self{'tz'};
    my $dmb = $$dmt{'base'};
-   my($gotmode,$type,@delta);
    $self->_init();
 
    if (! $instring) {
@@ -229,55 +309,67 @@ sub parse {
       return 1;
    }
 
-   my $mode  = $self->_rx('mode');
+   #
+   # Parse the string
+   #
 
- ENCODING: foreach my $string ($dmb->_encoding($instring)) {
+   $$self{'err'} = '';
+   $instring     =~ s/^\s*//;
+   $instring     =~ s/\s*$//;
 
-      $type      = 'delta';
+ PARSE: {
 
-      # Get the mode
+      # First, we'll try the standard format (without a mode string)
 
-      $gotmode  = 0;
-      $gotmode  = 1  if (defined($business));
-      $business = 0  if (! $business);
+      ($err,@delta) = $dmb->_split_delta($instring);
+      last PARSE  if (! $err);
 
-      if ($string =~ s/$mode//i) {
-         my $m = ($1);
-         if ($$dmb{'data'}{'wordmatch'}{'mode'}{lc($m)} == 1) {
-            $business = 0;
-         } else {
-            $business = 1;
+      # Next, we'll need to get a list of all the encodings and look
+      # for (and remove) the mode string from each.  We'll also recheck
+      # the standard format for each.
+
+      my @strings = $dmb->_encoding($instring);
+      my $moderx  = $self->_rx('mode');
+      my %mode    = ();
+
+      foreach my $string (@strings) {
+         if ($string =~ s/\s*$moderx\s*//i) {
+            my $b = $1;
+            if ($$dmb{'data'}{'wordmatch'}{'mode'}{lc($b)} == 1) {
+               $b = 0;
+            } else {
+               $b = 1;
+            }
+
+            ($err,@delta) = $dmb->_split_delta($string);
+            if (! $err) {
+               $business = $b;
+               $gotmode  = 1;
+               last PARSE;
+            }
+
+            $mode{$string} = $b;
          }
-         $gotmode = 1;
       }
 
-      $type         = 'business'  if ($business);
+      # Now we'll check each string for an expanded form delta.
 
-      # Parse the delta
-
-    PARSE: {
-
-         $$self{'err'} = '';
-         $string    =~ s/^\s*//;
-         $string    =~ s/\s*$//;
-
-         # Colon format
-
-         if ($string) {
-            my $tmp = $dmb->split($type,$string);
-            if (defined $tmp) {
-               @delta = @$tmp;
-               last ENCODING;
-            }
+      foreach my $string (@strings) {
+         my($b,$g);
+         if (exists $mode{$string}) {
+            $b = $mode{$string};
+            $g = 1;
+         } else {
+            $b = $business;
+            $g = 0;
          }
 
-         # Expanded format
+         my $past    = 0;
 
-         my $when      = $self->_rx('when');
-         my $past      = 0;
+         my $whenrx  = $self->_rx('when');
          if ($string  &&
-             $string =~ s/$when//i) {
-            my $when = ($1);
+             $string =~ s/$whenrx//i) {
+            my $when = $1;
             if ($$dmb{'data'}{'wordmatch'}{'when'}{lc($when)} == 1) {
                $past   = 1;
             }
@@ -286,45 +378,52 @@ sub parse {
          my $rx        = $self->_rx('expanded');
          if ($string  &&
              $string   =~ $rx) {
+            $business  = $b;
+            $gotmode   = $g;
             @delta     = @+{qw(y m w d h mn s)};
             foreach my $f (@delta) {
-               $f = 0  if (! defined $f);
-               $f =~ s/\s//g;
-            }
-            my $err;
-            if ($type eq 'business') {
-               ($err,@delta)  = $dmb->_normalize_business('split',@delta);
-            } else {
-               ($err,@delta)  = $dmb->_normalize_delta('split',@delta);
-            }
-
-            if ($err) {
-               next ENCODING;
+               if (! defined $f) {
+                  $f = 0;
+               } elsif (exists $$dmb{'data'}{'wordmatch'}{'nth'}{lc($f)}) {
+                  $f = $$dmb{'data'}{'wordmatch'}{'nth'}{lc($f)};
+               } else {
+                  $f =~ s/\s//g;
+               }
             }
 
             # if $past, reverse the signs
             if ($past) {
                foreach my $v (@delta) {
-                  if (defined $v) {
-                     $v *= -1;
-                  }
+                  $v *= -1;
                }
             }
 
-            last ENCODING;
+            last PARSE;
          }
       }
-
-      $$self{'err'} = "[parse] Invalid delta string";
-      last ENCODING;
    }
 
-   return 1  if ($$self{'err'});
+   if (! @delta) {
+      $$self{'err'} = "[parse] Invalid delta string";
+      return 1;
+   }
 
-   $$self{'data'}{'in'}       = $instring;
-   $$self{'data'}{'delta'}    = [@delta];
-   $$self{'data'}{'business'} = $business;
-   $$self{'data'}{'gotmode'}  = $gotmode;
+   ($err,@delta) = $dmb->_delta_fields( { 'nonorm'   => $no_normalize,
+                                          'business' => $business,
+                                          'source'   => 'string',
+                                          'sign'     => -1 },
+                                        [@delta]);
+
+   if ($err) {
+      $$self{'err'} = "[parse] Invalid delta string";
+      return 1;
+   }
+
+   $$self{'data'}{'in'}         = $instring;
+   $$self{'data'}{'delta'}      = [@delta];
+   $$self{'data'}{'business'}   = $business;
+   $$self{'data'}{'gotmode'}    = $gotmode;
+   $$self{'data'}{'length'}     = 'unknown';
    return 0;
 }
 
@@ -418,7 +517,8 @@ sub printf {
             if (! @field) {
                $out .= $match;
             } else {
-               $out .= $self->_printf_delta($sign,$pad,$width,$field[0],$field[$#field]);
+               $out .= $self->_printf_delta($sign,$pad,$width,$field[0],
+                                            $field[$#field]);
             }
 
          } else {
@@ -624,35 +724,16 @@ sub _printf_field_val {
    # Find the length of 1 unit of each field in terms of seconds.
 
    if (! exists $$self{'data'}{'flen'}{'s'}) {
-      $$self{'data'}{'flen'}{'s'} = 1;
-      $$self{'data'}{'flen'}{'m'} = 60;
-      $$self{'data'}{'flen'}{'h'} = 3600;
-
-      # Find the length of day/week/year
-      #
-      # $daylen is the number of second in a day
-      # $weeklen is the number of days in a week
-      # $yrlen is the number of days in a year
-
       my $business = $$self{'data'}{'business'};
-      my ($weeklen,$daylen,$yrlen);
-      if ($business) {
-         my $dmt = $$self{'tz'};
-         my $dmb = $$dmt{'base'};
-         $daylen  = $$dmb{'data'}{'calc'}{'bdlength'};
-         $weeklen = $$dmb{'data'}{'calc'}{'workweek'};
-         # The approximate length of the business year in business days
-         $yrlen   = 365.2425*$weeklen/7;
-      } else {
-         $weeklen = 7;
-         $daylen  = 86400;  # 24*60*60
-         $yrlen   = 365.2425;
-      }
-
-      $$self{'data'}{'flen'}{'d'} = $daylen;
-      $$self{'data'}{'flen'}{'w'} = $weeklen*$daylen;
-      $$self{'data'}{'flen'}{'M'} = $yrlen*$daylen/12;
-      $$self{'data'}{'flen'}{'y'} = $yrlen*$daylen;
+      my $dmb      = $self->base();
+      $$self{'data'}{'flen'} = { 's'  => 1,
+                                 'm'  => 60,
+                                 'h'  => 3600,
+                                 'd'  => $$dmb{'data'}{'len'}{$business}{'dl'},
+                                 'w'  => $$dmb{'data'}{'len'}{$business}{'wl'},
+                                 'M'  => $$dmb{'data'}{'len'}{$business}{'ml'},
+                                 'y'  => $$dmb{'data'}{'len'}{$business}{'yl'},
+                               };
    }
 
    # Calculate the value for each field.
@@ -685,24 +766,37 @@ sub _printf_field_val {
 
 sub type {
    my($self,$op) = @_;
+   $op = lc($op);
 
-   if ($op eq 'business') {
+   if      ($op eq 'business') {
       return $$self{'data'}{'business'};
+   } elsif ($op eq 'standard') {
+      return 1-$$self{'data'}{'business'};
+   }
 
-   } elsif ($op eq 'exact') {
-      my $exact = 1;
-      $exact    = 0  if ($$self{'data'}{'delta'}[0] != 0  ||
-                         $$self{'data'}{'delta'}[1] != 0  ||
-                         ($$self{'data'}{'delta'}[2] != 0  &&
-                          $$self{'data'}{'business'}));
+   my($exact,$semi,$approx) = (0,0,0);
+   my($y,$m,$w,$d,$h,$mn,$s) = @{ $$self{'data'}{'delta'} };
+   if ($y  ||  $m) {
+      $approx = 1;
+   } elsif ($w  ||  (! $$self{'data'}{'business'}  &&  $d)) {
+      $semi = 1;
+   } else {
+      $exact = 1;
+   }
+
+   if      ($op eq 'exact') {
       return $exact;
+   } elsif ($op eq 'semi') {
+      return $semi;
+   } elsif ($op eq 'approx') {
+      return $approx;
    }
 
    return undef;
 }
 
 sub calc {
-   my($self,$obj,$subtract) = @_;
+   my($self,$obj,$subtract,$no_normalize) = @_;
    if ($$self{'err'}) {
       $$self{'err'} = "[calc] First object invalid (delta)";
       return undef;
@@ -720,7 +814,7 @@ sub calc {
          $$self{'err'} = "[calc] Second object invalid (delta)";
          return undef;
       }
-      return $self->_calc_delta_delta($obj,$subtract);
+      return $self->_calc_delta_delta($obj,$subtract,$no_normalize);
 
    } else {
       $$self{'err'} = "[calc] Second object must be a Date/Delta object";
@@ -729,19 +823,30 @@ sub calc {
 }
 
 sub _calc_delta_delta {
-   my($self,$delta,$subtract) = @_;
+   my($self,$delta,@args) = @_;
    my $dmt = $$self{'tz'};
    my $dmb = $$dmt{'base'};
    my $ret = $self->new_delta;
 
    if ($self->err()) {
-      $$ret{'err'} = "[calc] Invalid delta/delta calculation object: delta1";
+      $$ret{'err'} = "[calc] First delta object invalid";
       return $ret;
    } elsif ($delta->err()) {
-      $$ret{'err'} = "[calc] Invalid delta/delta calculation object: delta2";
+      $$ret{'err'} = "[calc] Second delta object invalid";
       return $ret;
    }
 
+   my($subtract,$no_normalize);
+   if (@args == 2) {
+      ($subtract,$no_normalize) = @args;
+   } elsif ($args[0] eq 'nonormalize') {
+      $subtract     = 0;
+      $no_normalize = 1;
+   } else {
+      $subtract     = 0;
+      $no_normalize = 0;
+   }
+
    my $business = 0;
    if ($$self{'data'}{'business'} != $$delta{'data'}{'business'}) {
       $$ret{'err'} = "[calc] Delta/delta calculation objects must be of " .
@@ -751,22 +856,134 @@ sub _calc_delta_delta {
       $business = $$self{'data'}{'business'};
    }
 
-   my @delta;
+   my ($err,@delta);
    for (my $i=0; $i<7; $i++) {
       if ($subtract) {
          $delta[$i] = $$self{'data'}{'delta'}[$i] - $$delta{'data'}{'delta'}[$i];
       } else {
          $delta[$i] = $$self{'data'}{'delta'}[$i] + $$delta{'data'}{'delta'}[$i];
       }
-      $delta[$i] = "+" . $delta[$i]  if ($delta[$i] > 0);
    }
 
-   my $type  = ($business ? 'business' : 'delta');
-   $ret->set($type,\@delta);
+   ($err,@delta) = $dmb->_delta_fields( { 'nonorm'  => 0,
+                                          'source'  => 'delta',
+                                          'sign'    => -1 },
+                                        [@delta])  if (! $no_normalize);
+
+   $$ret{'data'}{'delta'}       = [@delta];
+   $$ret{'data'}{'business'}    = $business;
+   $$self{'data'}{'length'}     = 'unknown';
 
    return $ret;
 }
 
+sub convert {
+   my($self,$to) = @_;
+
+   # What mode are we currently in
+
+   my $from;
+   my($y,$m,$w,$d,$h,$mn,$s) = @{ $$self{'data'}{'delta'} };
+   if ($y  ||  $m) {
+      $from = 'approx';
+   } elsif ($w  ||  (! $$self{'data'}{'business'}  &&  $d)) {
+      $from = 'semi';
+   } else {
+      $from = 'exact';
+   }
+
+   my $business = $$self{'data'}{'business'};
+
+   #
+   # Do the conversion
+   #
+
+   {
+      no integer;
+
+      my $dmb = $self->base();
+      my $yl  = $$dmb{'data'}{'len'}{$business}{'yl'};
+      my $ml  = $$dmb{'data'}{'len'}{$business}{'ml'};
+      my $wl  = $$dmb{'data'}{'len'}{$business}{'wl'};
+      my $dl  = $$dmb{'data'}{'len'}{$business}{'dl'};
+
+      # Convert it to seconds
+
+      $s += $y*$yl + $m*$ml + $w*$wl + $d*$dl + $h*3600 + $mn*60;
+      ($y,$m,$w,$d,$h,$mn) = (0,0,0,0,0,0);
+
+      # Convert it to $to
+
+      if ($to eq 'approx') {
+         # Figure out how many months there are
+         $m          = int($s/$ml);
+         $s         -= $m*$ml;
+      }
+
+      if ($to eq 'approx'  ||  $to eq 'semi') {
+         if ($business) {
+            $w       = int($s/$wl);
+            $s      -= $w*$wl;
+         } else {
+            $d       = int($s/$dl);
+            $s      -= $d*$dl;
+         }
+      }
+
+      $s = int($s);
+   }
+
+   $self->set('delta',[$y,$m,$w,$d,$h,$mn,$s]);
+}
+
+sub cmp {
+   my($self,$delta) = @_;
+
+   if ($$self{'err'}) {
+      warn "WARNING: [cmp] Arguments must be valid deltas: delta1\n";
+      return undef;
+   }
+
+   if (! ref($delta) eq 'Date::Manip::Delta') {
+      warn "WARNING: [cmp] Argument must be a Date::Manip::Delta object\n";
+      return undef;
+   }
+   if ($$delta{'err'}) {
+      warn "WARNING: [cmp] Arguments must be valid deltas: delta2\n";
+      return undef;
+   }
+
+   if ($$self{'data'}{'business'} != $$delta{'data'}{'business'}) {
+      warn "WARNING: [cmp] Deltas must both be business or standard\n";
+      return undef;
+   }
+
+   my $business = $$self{'data'}{'business'};
+   my $dmb      = $self->base();
+   my $yl       = $$dmb{'data'}{'len'}{$business}{'yl'};
+   my $ml       = $$dmb{'data'}{'len'}{$business}{'ml'};
+   my $wl       = $$dmb{'data'}{'len'}{$business}{'wl'};
+   my $dl       = $$dmb{'data'}{'len'}{$business}{'dl'};
+
+   if ($$self{'data'}{'length'} eq 'unknown') {
+      my($y,$m,$w,$d,$h,$mn,$s) = @{ $$self{'data'}{'delta'} };
+
+      no integer;
+      $$self{'data'}{'length'}  = int($y*$yl + $m*$ml + $w*$wl +
+                                      $d*$dl + $h*3600 + $mn*60 + $s);
+   }
+
+   if ($$delta{'data'}{'length'} eq 'unknown') {
+      my($y,$m,$w,$d,$h,$mn,$s) = @{ $$delta{'data'}{'delta'} };
+
+      no integer;
+      $$delta{'data'}{'length'}  = int($y*$yl + $m*$ml + $w*$wl +
+                                       $d*$dl + $h*3600 + $mn*60 + $s);
+   }
+
+   return ($$self{'data'}{'length'} cmp $$delta{'data'}{'length'});
+}
+
 1;
 # Local Variables:
 # mode: cperl
@@ -776,5 +993,5 @@ sub _calc_delta_delta {
 # cperl-continued-brace-offset: 0
 # cperl-brace-offset: 0
 # cperl-brace-imaginary-offset: 0
-# cperl-label-offset: -2
+# cperl-label-offset: 0
 # End:
