@@ -1,5 +1,5 @@
 package Date::Manip::TZ;
-# Copyright (c) 2008-2010 Sullivan Beck. All rights reserved.
+# Copyright (c) 2008-2013 Sullivan Beck. All rights reserved.
 # This program is free software; you can redistribute it and/or modify it
 # under the same terms as Perl itself.
 
@@ -24,9 +24,23 @@ require Date::Manip::Zones;
 use Date::Manip::Base;
 
 our $VERSION;
-$VERSION='6.14';
+$VERSION='6.39';
 END { undef $VERSION; }
 
+# To get rid of a 'used only once' warnings.
+END {
+   my $tmp = \%Date::Manip::Zones::Module;
+   $tmp    = \%Date::Manip::Zones::ZoneNames;
+   $tmp    = \%Date::Manip::Zones::Alias;
+   $tmp    = \%Date::Manip::Zones::Abbrev;
+   $tmp    = \%Date::Manip::Zones::Offmod;
+   $tmp    = $Date::Manip::Zones::FirstDate;
+   $tmp    = $Date::Manip::Zones::LastDate;
+   $tmp    = $Date::Manip::Zones::LastYear;
+   $tmp    = $Date::Manip::Zones::TzcodeVersion;
+   $tmp    = $Date::Manip::Zones::TzdataVersion;
+}
+
 ########################################################################
 # BASE METHODS
 ########################################################################
@@ -57,6 +71,8 @@ sub _init {
 
       # methods     a list of methods used for determining the
       #             current zone
+      # path        the PATH to set for determining the current
+      #             zone
       # dates       critical dates on a per/year (UT) basis
       # zonerx      the regular expression for matching timezone
       #             names/aliases
@@ -67,6 +83,7 @@ sub _init {
       # zrx         the regular expression to match all timezone
       #             information
       'methods'        => [],
+      'path'           => undef,
       'zonerx'         => undef,
       'abbrx'          => undef,
       'offrx'          => undef,
@@ -79,9 +96,10 @@ sub _init {
    my $os  = $dmb->_os();
 
    if ($os eq 'Unix') {
+      $$self{'data'}{'path'}    = '/bin:/usr/bin';
       $$self{'data'}{'methods'} = [
                                    qw(main TZ
-                                      env  TZ
+                                      env  zone TZ
                                       file /etc/TIMEZONE
                                       file /etc/timezone
                                       file /etc/sysconfig/clock
@@ -103,7 +121,7 @@ sub _init {
    } elsif ($os eq 'Windows') {
       $$self{'data'}{'methods'} = [
                                    qw(main TZ
-                                      env  TZ
+                                      env  zone TZ
                                       registry
                                       gmtoff),
                                   ];
@@ -111,13 +129,12 @@ sub _init {
    } elsif ($os eq 'VMS') {
       $$self{'data'}{'methods'} = [
                                    qw(main TZ
-                                      env  TZ
-                                      env  SYS$TIMEZONE_RULE
-                                      env  SYS$TIMEZONE_NAME
-                                      env  UCX$TZ
-                                      env  TCPIP$TZ
-                                      env  MULTINET_TIMEZONE
-                                      env  SYS$TIMEZONE_DIFFERENTIAL
+                                      env  zone TZ
+                                      env  zone SYS$TIMEZONE_NAME
+                                      env  zone UCX$TZ
+                                      env  zone TCPIP$TZ
+                                      env  zone MULTINET_TIMEZONE
+                                      env  offset SYS$TIMEZONE_DIFFERENTIAL
                                       gmtoff
                                     ),
                                   ];
@@ -125,7 +142,7 @@ sub _init {
    } else {
       $$self{'data'}{'methods'} = [
                                    qw(main TZ
-                                      env  TZ
+                                      env  zone TZ
                                       gmtoff
                                     ),
                                   ];
@@ -384,7 +401,13 @@ sub _get_curr_zone {
    my $dstflag = ($isdst ? 'dstonly' : 'stdonly');
 
    my (@methods) = @{ $$self{'data'}{'methods'} };
- METHOD: while (@methods) {
+   my $debug     = ($ENV{DATE_MANIP_DEBUG} ? 1 : 0);
+
+   defined $$self{'data'}{'path'}
+     and local $ENV{PATH} = $$self{'data'}{'path'};
+
+   METHOD:
+   while (@methods) {
       my $method = shift(@methods);
       my @zone   = ();
 
@@ -397,13 +420,44 @@ sub _get_curr_zone {
          my $var = shift(@methods);
          push(@zone,$$::var)  if (defined $$::var);
 
+         if ($debug) {
+            print "*** DEBUG ***  main $var = " .
+                  (defined $$::var ? $$::var : 'undef') . "\n";
+         }
+
       } elsif ($method eq 'env') {
-         if (! @methods) {
-            warn "ERROR: [_set_curr_zone] env requires argument\n";
+         if (@methods < 2) {
+            warn "ERROR: [_set_curr_zone] env requires 2 argument\n";
             return;
          }
-         my $var = shift(@methods);
-         push(@zone,$ENV{$var})  if (exists $ENV{$var});
+         my $type = lc( shift(@methods) );
+         if ($type ne 'zone'  &&
+             $type ne 'offset') {
+            warn "ERROR: [_set_curr_zone] env requires 'offset' or 'zone' as the first argument\n";
+            return;
+         }
+         my $var  = shift(@methods);
+         if (exists $ENV{$var}) {
+            if ($type eq 'zone') {
+               push(@zone,$ENV{$var});
+            } else {
+               my $off = $ENV{$var};
+               $off    = $dmb->_delta_convert('time',"0:0:$off");
+               $off    = $dmb->_delta_convert('offset',$off);
+               push(@zone,$off);
+            }
+         }
+
+         if ($debug) {
+            print "*** DEBUG *** env $type $var ";
+            if (exists $ENV{$var}) {
+               print $ENV{$var};
+               print $zone[$#zone]  if ($type eq 'offset');
+               print "\n";
+            } else {
+               print "-no result-\n";
+            }
+         }
 
       } elsif ($method eq 'file') {
          if (! @methods) {
@@ -416,25 +470,81 @@ sub _get_curr_zone {
          my $in = new IO::File;
          $in->open($file)  ||  next;
          my $firstline = 1;
+
+         my @z;
          while (! $in->eof) {
             my $line = <$in>;
             next  if ($line =~ /^\s*\043/  ||
                       $line =~ /^\s*$/);
-            if ($line =~ /^\s*(?:TZ|TIMEZONE|ZONE)\s*=\s*(\S+)/i) {
-               my $zone = $1;
-               $zone    =~ s/["']//g;  # "
-               push(@zone,$zone);
+
+            # We're looking for lines of the form:
+            #   TZ = string
+            #   TIMEZONE = string
+            #   ZONE = string
+            #
+            # 'string' can be:
+            #   the name of a timezone enclosed in single/double quotes
+            #   with everything after the closing quote ignored (the
+            #   name of the timezone may have spaces instead of underscores)
+            #
+            #   a space delimited list of tokens, the first of which
+            #   is the time zone
+            #
+            #   the name of a timezone with underscores replaced by
+            #   spaces and nothing after the timezone
+            #
+            # For some reason, RHEL6 desktop version stores timezones as
+            #   America/New York
+            # instead of
+            #   America/New_York
+            # which is why we have to handle the space/underscore
+            # substitution.
+
+            if ($line =~ /^\s*(?:TZ|TIMEZONE|ZONE)\s*=\s*(.*)\s*$/) {
+               my $val  = $1;
+               @z       = ();
+               last  if (! $val);
+
+               if ($val =~ /^(["'])(.*?)\1/) {
+                  my $z = $2;
+                  last  if (! $z);
+                  $z    =~ s/\s+/_/g;
+                  push(@zone,$z);
+
+               } elsif ($val =~ /\s/) {
+                  $val  =~ /^(\S+)/;
+                  push(@zone,$1);
+                  $val  =~ s/\s+/_/g;
+                  push(@zone,$val);
+
+               } else {
+                  push(@zone,$val);
+               }
+
                last;
             }
             if ($firstline) {
                $firstline = 0;
-               $line      =~ s/\s//g;
+               $line      =~ s/^\s*//;
+               $line      =~ s/\s*$//;
                $line      =~ s/["']//g;  # "
-               push(@zone,$line);
+               $line      =~ s/\s+/_/g;
+               push(@z,$line);
             }
          }
          close(IN);
 
+         push(@zone,@z)  if (@z);
+
+         if ($debug) {
+            print "*** DEBUG *** file $file\n";
+            if (@z) {
+               print "              @z\n";
+            } else {
+               print "              -no result-\n";
+            }
+         }
+
       } elsif ($method eq 'command') {
          if (! @methods) {
             warn "ERROR: [_set_curr_zone] command requires argument\n";
@@ -444,6 +554,15 @@ sub _get_curr_zone {
          my ($out)   = _cmd($command);
          push(@zone,$out)  if ($out);
 
+         if ($debug) {
+            print "*** DEBUG *** command $command\n";
+            if ($out) {
+               print "              $out\n";
+            } else {
+               print "              -no result-\n";
+            }
+         }
+
       } elsif ($method eq 'cmdfield') {
          if ($#methods < 1) {
             warn "ERROR: [_set_curr_zone] cmdfield requires 2 arguments\n";
@@ -452,11 +571,24 @@ sub _get_curr_zone {
          my $command = shift(@methods);
          my $n       = shift(@methods);
          my ($out)   = _cmd($command);
+         my @z;
+
          if ($out) {
             $out    =~ s/^\s*//;
             $out    =~ s/\s*$//;
             my @out = split(/\s+/,$out);
-            push(@zone,$out[$n])  if (defined $out[$n]);
+            push(@z,$out[$n])  if (defined $out[$n]);
+         }
+
+         push(@zone,@z)  if (@z);
+
+         if ($debug) {
+            print "*** DEBUG *** cmdfield $command $n\n";
+            if (@z) {
+               print "              @z\n";
+            } else {
+               print "              -no result-\n";
+            }
          }
 
       } elsif ($method eq 'gmtoff') {
@@ -477,10 +609,18 @@ sub _get_curr_zone {
          $off    = $dmb->_delta_convert('offset',$off);
          push(@zone,$off);
 
+         if ($debug) {
+            print "*** DEBUG *** gmtoff $off\n";
+         }
+
       } elsif ($method eq 'registry') {
          my $z = $self->_windows_registry_val();
          push(@zone,$z)  if ($z);
 
+         if ($debug) {
+            print "*** DEBUG *** registry $z\n";
+         }
+
       } else {
          warn "ERROR: [_set_curr_zone] invalid method: $method\n";
          return;
@@ -773,7 +913,9 @@ sub zone {
          my $isdst = '';
          $isdst    = 0  if ($dstflag eq 'stdonly');
          $isdst    = 1  if ($dstflag eq 'dstonly');
-       ZONE: foreach my $z (@zone) {
+
+         ZONE:
+         foreach my $z (@zone) {
             $self->_module($z);
             my $y       = $$date[0];
             my @periods = $self->_all_periods($z,$y);
@@ -818,7 +960,8 @@ sub _check_abbrev_isdst {
    my($self,$abbrev,$isdst,@zones) = @_;
 
    my @ret;
- ZONE: foreach my $zone (@zones) {
+   ZONE:
+   foreach my $zone (@zones) {
       $self->_module($zone);
 
       foreach my $y (sort keys %{ $$self{'data'}{'Zones'}{$zone}{'Dates'} }) {
@@ -1053,7 +1196,7 @@ sub date_period {
    my $dmb  = $$self{'base'};
    my @date = @$date;
    my $year = $date[0];
-   my $dates= $dmb->_join_date($date);
+   my $dates= $dmb->_date_fields(@$date);
 
    if ($wallclock) {
       # A wallclock date
@@ -1208,7 +1351,7 @@ sub convert_to_gmt {
    my $dmb = $$self{'base'};
 
    if (! $from) {
-      ($from) = $self->_now('tz',1);
+      $from = $self->_now('tz',1);
    }
    $self->_convert('convert_to_gmt',$date,$from,'GMT',$isdst);
 }
@@ -1221,7 +1364,7 @@ sub convert_from_gmt {
    my $dmb = $$self{'base'};
 
    if (! $to) {
-      ($to) = $self->_now('tz',1);
+      $to = $self->_now('tz',1);
    }
    $self->_convert('convert_from_gmt',$date,'GMT',$to,$isdst);
 }
@@ -1409,7 +1552,7 @@ sub _offrx {
                                   $hr
                               )
                  )
-                 (?: \s* \( $abb \))? /ix;
+                 (?: \s* (?: \( $abb \) | $abb))? /ix;
 
    $$self{'data'}{'offrx'} = $re;
    return $$self{'data'}{'offrx'};
@@ -1431,7 +1574,7 @@ sub _zrx {
    my $zoneabbrx = $self->_abbrx();           # (?<abb>edt|est|...)
    my $zoneoffrx = $self->_offrx();           # (?<off>07:00) (?<abb>GMT)
 
-   my $zrx       = qr/(?<tzstring>$zonerx|$zoneabbrx|$zoneoffrx)/;
+   my $zrx       = qr/(?<tzstring>$zoneabbrx|$zoneoffrx|$zonerx)/;
    $$self{'data'}{'zrx'} = $zrx;
    return $zrx;
 }
@@ -1471,6 +1614,10 @@ sub _config_var_tz {
       return  if ($err);
       $$self{'data'}{'sections'}{'conf'}{'setdate'} = 0;
       $val = 1;
+
+   } elsif ($var eq 'configfile') {
+      $self->_config_file($val);
+      return;
    }
 
    my $base = $$self{'base'};
@@ -1482,8 +1629,8 @@ sub _config_var_setdate {
    my($self,$val,$force) = @_;
    my $base = $$self{'base'};
 
-   my $dstrx = qr/(?:,(stdonly|dstonly|std|dst))?/i;
-   my $zonrx = qr/,(.+)/;
+   my $dstrx = qr/(?:,\s*(stdonly|dstonly|std|dst))?/i;
+   my $zonrx = qr/,\s*(.+)/;
    my $da1rx = qr/(\d\d\d\d)(\d\d)(\d\d)(\d\d):(\d\d):(\d\d)/;
    my $da2rx = qr/(\d\d\d\d)\-(\d\d)\-(\d\d)\-(\d\d):(\d\d):(\d\d)/;
    my $time  = time;
@@ -1529,7 +1676,7 @@ sub _config_var_setdate {
       $op = 'date';
       my($y,$m,$d,$h,$mn,$s) = ($1,$2,$3,$4,$5,$6);
       $date   = [$y,$m,$d,$h,$mn,$s];
-      ($zone) = $self->_now('systz',1);
+      $zone   = $self->_now('systz',1);
 
    } elsif (lc($val) eq 'now') {
       # now
@@ -1580,7 +1727,7 @@ sub _config_var_setdate {
 
       $zone = $self->zone(@args);
       if (! $zone) {
-         warn "ERROR: [config_var] invalid zone in SetDate\n";
+         warn "ERROR: [config_var] invalid zone in SetDate: @args\n";
          return 1;
       }
 
@@ -1615,7 +1762,7 @@ sub _config_var_setdate {
       }
 
       if (! $per) {
-         warn "ERROR: [config_var] invalid date: SetDate\n";
+         warn "ERROR: [config_var] invalid date: SetDate: $date, $zone\n";
          return 1;
       }
       $isdst  = $$per[5];
@@ -1628,7 +1775,7 @@ sub _config_var_setdate {
       my($err);
       ($err,$date,$offset,$isdst,$abb) = $self->convert_from_gmt($date,$zone);
       if ($err) {
-         warn "ERROR: [config_var] invalid SetDate date/offset values\n";
+         warn "ERROR: [config_var] invalid SetDate date/offset values: $date, $zone\n";
          return 1;
       }
    }
@@ -1670,5 +1817,5 @@ sub _config_var_setdate {
 # cperl-continued-brace-offset: 0
 # cperl-brace-offset: 0
 # cperl-brace-imaginary-offset: 0
-# cperl-label-offset: -2
+# cperl-label-offset: 0
 # End:
